//+------------------------------+
//|   CPU_Utilization_Monitor    |
//+==============================+
//|
//|   Rev Date: 2011.02.14
//|
//|   Contains the following Functions:
//|
//|   Processor_Count()
//|   CPU_Utilization()
//|   Available_Processors()
//|
//+------------------------------+

#property copyright "Copyright  2011, 1005phillip"

#import "kernel32.dll"
   // GetSystemTimes documented here: http://msdn.microsoft.com/en-us/library/ms724400(v=vs.85).aspx
   bool GetSystemTimes(int & lpIdleTime[], int & lpKernelTime[], int & lpUserTime[]);
   // GetEnvironmentVariable documented here: http://msdn.microsoft.com/en-us/library/ms683188(VS.85).aspx
   int GetEnvironmentVariableA(string lpName, string lpBuffer, int nSize);
#import

//+------------------------------+
//|   Processor_Count()          |
//|==============================+
//|   int Processor_Count(bool refresh_count=false, bool verbose=false)
//|   
//|   Determines the total number of processors accessible by the operating system on the local computer.
//|   Returns an integer value representing the processor count if possible, otherwise it returns a -1 value.
//|   
//|   Parameters:
//|   
//|      refresh_count  -  Determines whether or not the processor count is recomputed on successive calls to this function.
//|                        By default, successive calls to Processor_Count() will simply return the previously determined processor count for the system.
//|      verbose        -  Determines whether or not print messages are committed to the experts log. By default, print messages are disabled.
//|   
//|   Sample:
//|   
//|      int Current_Processor_Count=Processor_Count();  // call this function to return the processor count for the system
//|      if(Current_Processor_Count<1) Print("Processor_Count() call routine could not determine the processor count for this system");
//|      else if(Current_Processor_Count==1) Print("The Operating System reports there is ",Current_Processor_Count," processor present in the local computer");
//|      else if(Current_Processor_Count>1) Print("The Operating System reports there are ",Current_Processor_Count," processors present in the local computer");
//|   
//+------------------------------+
int Processor_Count(bool refresh_count=false, bool verbose=false)
   {  // Processor_Count body start
   int      result=0;
   string   Processor_Count_String="01234567890123456789";
   
   static int Current_Processor_Count=-1;  // define the processor count to be -1 (the default error value) until a successful query has been completed
   
   // if processor count is not zero and the refresh_count flag is set to false then no need to recompute the processor count, simply return the previously determined value
   if(Current_Processor_Count>0 && refresh_count==false)
      {
      if(verbose==true)
         {
         Print("");
         Print("Processor_Count(): Processor_Count was previously computed since EA was started, no need to reinitialize, returning pre-determined processor count");
         Print("");
         Print("Processor_Count(): Processor_Count = ",Current_Processor_Count);
         }
      return(Current_Processor_Count);
      }
   
   // GetEnvironmentVariable documented here: http://msdn.microsoft.com/en-us/library/ms683188(VS.85).aspx
   result = GetEnvironmentVariableA("NUMBER_OF_PROCESSORS",Processor_Count_String, StringLen(Processor_Count_String)); // get the processor count for this computer
   Current_Processor_Count=StrToInteger(Processor_Count_String); // convert string to integer
   
   if(result==0 || Current_Processor_Count<1)
      {
      // If the function fails, the return value is zero. If the specified environment variable was not found in the environment block, GetLastError returns ERROR_ENVVAR_NOT_FOUND.
      // If lpBuffer is not large enough to hold the data, the return value is the buffer size, in characters, required to hold the string and its terminating null character and the contents of lpBuffer are undefined.
      Print("Processor_Count(): Processor count was not determined, an internal error has occurred");
      return(-1);  // return the -1 value which is to be interpretted by the calling routine as an error having occurred in this specific call function
      }
   
   if(verbose==true)
      {
      Print("");
      if(Current_Processor_Count==1) Print("Processor_Count(): The Operating System reports there is ",Current_Processor_Count," processor present in the local computer");
      else Print("Processor_Count(): The Operating System reports there are ",Current_Processor_Count," processors present in the local computer");
      }
   
   return(Current_Processor_Count);
   
   }  // Processor_Count body end

// End Call Function: Processor_Count()

//+------------------------------+
//|   CPU_Utilization()          |
//|==============================+
//|   double CPU_Utilization(int Dwell_time=1000, bool verbose=false)
//|   
//|   Computes the average CPU utilization for the local computer over the duration of the specific sampling time (1 second by default).
//|   Returns a double value representing the CPU utilization as a fraction (1 = 100% utilization, 0.5 = 50% utilization) if possible, otherwise, it returns a value of -1.
//|   
//|   Parameters:
//|
//|      Dwell_time  -  Dwell time in milliseconds over which the function collects utilization statistics. By default, the dwell time is 1000ms (1 second).
//|                     Longer dwell times (>5000ms) return utilization values that are less prone to background fluctuations in processor utilization at the expense of taking longer to compute the result.
//|                     Shorter dwell times (<5000ms) return utilization values that are more prone to background fluctuations in processor utilization but take less time to compute the result.
//|      verbose     -  Determines whether Print messages are committed to the experts log. By default, print messages are disabled.
//|   
//|   Sample:
//|   
//|      double Current_CPU_Utilization=CPU_Utilization();  // call this function to return the current CPU utilization rate for the system
//|      if(MathFloor(Current_CPU_Utilization)<0) Print("CPU_Utilization() call routine could not determine the CPU utilization rate for this system");
//|      else Print("CPU Utilization = ",MathFloor(Current_CPU_Utilization*100.)," %");
//|   
//+------------------------------+
double CPU_Utilization(int Dwell_time=1000, bool verbose=false)
   {  // CPU_Utilization body start
   int      idle_Initial[1], idle_Final[1], kernel_Initial[1], kernel_Final[1], user_Initial[1], user_Final[1];
   double   Current_CPU_Utilization;
   bool     result;
   // GetSystemTimes documented here: http://msdn.microsoft.com/en-us/library/ms724400(v=vs.85).aspx
   
   result=GetSystemTimes(idle_Initial, kernel_Initial, user_Initial);  // get the initial system times
   if(result==false)
      {
      // If the function fails, the return value is zero.
      Print("");
      Print("CPU_Utilization(): CPU utilization could not be determined, an internal error has occurred");
      return(-1);  // return the -1 value which is to be interpretted by the calling routine as an error having occurred in this specific call function
      }
   
   Sleep(Dwell_time);  // sleep
   
   result=GetSystemTimes(idle_Final, kernel_Final, user_Final);  // get the final system times after sleeping
   if(result==false)
      {
      // If the function fails, the return value is zero.
      Print("");
      Print("CPU_Utilization(): CPU utilization could not be determined, an internal error has occurred");
      return(-1);  // return the -1 value which is to be interpretted by the calling routine as an error having occurred in this specific call function
      }
   
   //kernel time includes the amount of time the system has been idle, per definition in GetSystemTimes Function
   
   Current_CPU_Utilization=(kernel_Final[0]-kernel_Initial[0] + user_Final[0]-user_Initial[0]-(idle_Final[0]-idle_Initial[0]))*1./(1.*(kernel_Final[0]-kernel_Initial[0] + user_Final[0]-user_Initial[0]));
   
   if(verbose==true)
      {
      Print("");
      Print("CPU_Utilization(): CPU Utilization = ",MathFloor(Current_CPU_Utilization*100.)," %");
      }
   
   return(Current_CPU_Utilization);
   
   }  // CPU_Utilization body end

// End Call Function: CPU_Utilization()

//+------------------------------+
//|   Available_Processors()     |
//|==============================+
//|   int Available_Processors(int Dwell_time=1000, bool refresh_count=false, bool verbose=false)
//|   
//|   Computes the number of available (currently unused) processors in the local computer.
//|   Returns an integer value representing the number of unused processors, otherwise, it returns a value of -1.
//|   
//|   Parameters:
//|
//|      Dwell_time     -  Dwell time in milliseconds over which the function collects utilization statistics. By default, the dwell time is 1000ms (1 second).
//|                        Longer dwell times (>5000ms) return utilization values that are less prone to background fluctuations in processor utilization at the expense of taking longer to compute the result.
//|                        Shorter dwell times (<5000ms) return utilization values that are more prone to background fluctuations in processor utilization but take less time to compute the result.
//|      refresh_count  -  Determines whether or not the processor count is recomputed on successive calls to this function
//|      verbose        -  Determines whether Print messages are committed to the experts log. By default, print messages are disabled.
//|   
//|   Sample:
//|   
//|      int Current_Available_Processors=Available_Processors();  // call this function to return the number of available processors for the system
//|      if(Current_Available_Processors==-1) Print("Available_Processors() call routine could not determine the number of unused processors for this system");
//|      else if(Current_Available_Processors==0) Print("There are no available processors at this time, the system is fully loaded");
//|      else if(Current_Available_Processors==1) Print("There is ",Current_Available_Processors," processor available for additional processing on this system");
//|      else if(Current_Available_Processors>1) Print("There are ",Current_Available_Processors," processors available for additional processing on this system");
//|   
//+------------------------------+
int Available_Processors(int Dwell_time=1000, bool refresh_count=false, bool verbose=false)
   {  // Available_Processors body start
   
   // Step 1: Retrieve the processor count value for the local computer
   int Current_Processor_Count=Processor_Count(refresh_count,verbose);
   
   if(Current_Processor_Count<1) // error checking
      {
      Print("");
      Print("Available_Processors(): Processor_Count() call routine could not determine the processor count for this system");
      return(-1);  // return the -1 value which is to be interpretted by the calling routine as an error having occurred in this specific call function
      }
   
   if(verbose==true)
      {
      Print("");
      if(Current_Processor_Count==1) Print("Available_Processors(): The Operating System reports there is ",Current_Processor_Count," processor present in the local computer");
      else Print("Available_Processors(): The Operating System reports there are ",Current_Processor_Count," processors present in the local computer");
      }
   
   // Step 2: Retrieve the CPU utilization rate the local computer
   double Current_CPU_Utilization=CPU_Utilization(Dwell_time,verbose);
   
   if(MathFloor(Current_CPU_Utilization)<0) // error checking
      {
      Print("");
      Print("Available_Processors(): CPU_Utilization() call routine could not determine the CPU utilization rate for this system");
      return(-1);  // return the -1 value which is to be interpretted by the calling routine as an error having occurred in this specific call function
      }
   
   if(verbose==true)
      {
      Print("");
      Print("Available_Processors(): CPU Utilization = ",MathFloor(Current_CPU_Utilization*100.)," %");
      }
   
   // Step 3: Compute the remaining number of processors that are currently unused in the local computer
   
   int Current_Available_Processors = MathFloor(1.*Current_Processor_Count*(1.-Current_CPU_Utilization).);
   
   if(verbose==true)
      {
      Print("");
      if(Current_Available_Processors==0) Print("Available_Processors(): There are no available processors at this time, the system is fully loaded");
      else if(Current_Available_Processors==1) Print("Available_Processors(): There is ",Current_Available_Processors," processor available for additional processing on this system");
      else if(Current_Available_Processors>1) Print("Available_Processors(): There are ",Current_Available_Processors," processors available for additional processing on this system");
      }
   
   return(Current_Available_Processors);
   
   }  // Available_Processors body end

// End Call Function: Available_Processors()

// Program End

/*
   +------------------------------------------------------------------+
   |                                                                  |
   |   Revision History                                               |
   |                                                                  |
   +------------------------------------------------------------------+

 Objective: This include file contains the routines necessary for computing the total processor count for the local computer, the current CPU utilization rate of those processors,
            and the number of unused (i.e. available) processors.
            
 Future Work:  None planned at the time of the current revision date.
            
 2011 Worklog:
   
   Feb 14   Fixed an error in the free processors calculation
   
   Feb 13   Implementing Available_Processors() call function, intended use are to ensure that CPU intensive tasks are not spawned by EA's, scripts,
            or indicators until such time that the proper CPU resources are available so as to better manage potential resource conflicts.
   
   Feb 12   Finished up initial implementation of the Processor_Count() call function...corrected the type-casting errors highlighted by MQL4 forum member "jjc".
   
   Feb 11   Implementing Processor_Count() call function, significant contributions to the successful implementation of these algorithms are credited to MQL4 forum members "WHRoeder" and "jjc".
            see: http://forum.mql4.com/38452/page2
   
   Jan 26   Implementing CPU_Utilization() call function, significant contributions to the successful implementation of these algorithms are credited to MQL4 forum member "qjol".
            see: http://forum.mql4.com/38452
   
*/


